"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const PURE_ANNOTATION = "#__PURE__";
const isPureAnnotated = (comments) => {
    if (comments && comments.length > 0) {
        return comments[comments.length - 1].value === PURE_ANNOTATION;
    }
    return false;
};
const containsNodePath = (nodePaths, nodePath) => {
    return nodePaths.indexOf(nodePath) > -1;
};
const createComponentBlock = (value) => ({
    type: "CommentBlock",
    value,
});
const isDeclaredVariableUsedInCallback = (nodePath) => {
    let used = false;
    if (nodePath.parentPath.isVariableDeclarator()) {
        const scope = nodePath.parentPath.scope;
        const id = nodePath.parentPath.get("id");
        const traverseReferencedIdentifier = (np) => np.traverse({
            ReferencedIdentifier(n) {
                if (scope.hasOwnBinding(n.node.name)) {
                    if (id.isIdentifier()) {
                        if (id.node.name === n.node.name) {
                            used = true;
                        }
                    }
                }
            },
        });
        nodePath.traverse({
            ArrowFunctionExpression: (nodePath) => {
                traverseReferencedIdentifier(nodePath);
            },
            FunctionExpression: (nodePath) => {
                traverseReferencedIdentifier(nodePath);
            },
        });
    }
    return used;
};
const isPureCall = (nodePath) => {
    // x = pureCall()
    // a.x = pureCall()
    if (nodePath.parentPath.isAssignmentExpression()) {
        return true;
    }
    // [pureCall()]
    if (nodePath.parentPath.isArrayExpression()) {
        return true;
    }
    // ({ x: prueCall(0,1) })
    if (nodePath.parentPath.isProperty()) {
        return true;
    }
    // const x = pureCall()
    if (nodePath.parentPath.isVariableDeclarator()) {
        return !isDeclaredVariableUsedInCallback(nodePath);
    }
    // otherCall(pureCall())
    if (nodePath.parentPath.isCallExpression()) {
        return containsNodePath(nodePath.parentPath.get("arguments") || [], nodePath);
    }
    return false;
};
exports.default = () => ({
    name: "pure-calls-annotation",
    inherits: require("@babel/plugin-syntax-dynamic-import").default,
    visitor: {
        CallExpression: {
            enter(nodePath) {
                if (isPureCall(nodePath)) {
                    if (!isPureAnnotated(nodePath.node.leadingComments)) {
                        const pureAnnotation = createComponentBlock(PURE_ANNOTATION);
                        nodePath.replaceWith(Object.assign(Object.assign({}, nodePath.node), { leadingComments: nodePath.node.leadingComments
                                ? nodePath.node.leadingComments.concat(pureAnnotation)
                                : [pureAnnotation] }));
                    }
                }
            },
        },
    },
});
//# sourceMappingURL=index.js.map